@@ -8,15 +8,14 @@ from warnings import warn
 from astropy.coordinates import Angle, frame_transform_graph
 import astropy.units as u
 from astropy.utils.data import get_readable_fileobj
-from astropy.utils.decorators import deprecated
 
 from ...core import Regions
 from ...core.registry import RegionsRegistry
-from ..core import _Shape, _ShapeList, reg_mapping
+from .io_core import _Shape, _ShapeList, reg_mapping
 from .core import (CRTFRegionParserError, CRTFRegionParserWarning,
                    valid_symbols)
 
-__all__ = ['read_crtf', 'CRTFParser']
+__all__ = []
 
 # All CASA files start with '#CRTF' . It may also include the version
 # number like '#CRTFv0' .
@@ -424,7 +423,7 @@ class _CRTFRegionParser:
                     val2 = par[3]
                 val1 = val1.strip()
                 val2 = val2.strip()
-                if val1 in CRTFParser.valid_global_keys or val1 == 'label':
+                if val1 in _CRTFParser.valid_global_keys or val1 == 'label':
                     if val1 in ('range', 'corr', 'labeloff'):
                         val2 = val2.split(',')
                         val2 = [x.strip() for x in val2]
@@ -517,56 +516,3 @@ class _CRTFCoordinateParser:
         else:
             raise CRTFRegionParserError('Units must be specified for '
                                         f'{string_rep}')
-
-
-@deprecated('0.5', alternative='`regions.Regions.read`')
-def read_crtf(filename, errors='strict', cache=False):
-    """
-    Read a CRTF region file and return a list of region objects.
-
-    Parameters
-    ----------
-    filename : str
-        The filename of the file to access.
-
-    errors : {'strict', 'warn', 'ignore'}, optional
-        The error handling scheme to use for handling parsing
-        errors. The default is 'strict', which will raise a
-        `~regions.CRTFRegionParserError`. 'warn' will raise a
-        `~regions.CRTFRegionParserWarning`, and 'ignore' will do nothing
-        (i.e., be silent).
-
-    cache : bool or 'update', optional
-        Whether to cache the contents of remote URLs. If 'update', check
-        the remote URL for a new version but store the result in the
-        cache.
-
-    Returns
-    -------
-    regions : list
-        A list of `~regions.Region` objects.
-    """
-    return _read_crtf(filename, errors=errors, cache=cache)
-
-
-@deprecated('0.5', alternative='`regions.Regions.parse`')
-class CRTFParser(_CRTFParser):
-    """
-    Parse a CRTF string.
-
-    This class transforms a CRTF string to a
-    `~regions.io.core.ShapeList`. The result is stored in the ``shapes``
-    attribute.
-
-    Parameters
-    ----------
-    region_string : str
-        A CRTF region string.
-
-    errors : {'strict', 'warn', 'ignore'}, optional
-        The error handling scheme to use for handling parsing
-        errors. The default is 'strict', which will raise a
-        `~regions.CRTFRegionParserError`. 'warn' will raise a
-        `~regions.CRTFRegionParserWarning`, and 'ignore' will do nothing
-        (i.e., be silent).
-    """
