@@ -10,9 +10,10 @@ import astropy.units as u
 from astropy.wcs.utils import pixel_to_skycoord
 import numpy as np
 
-from ..core.attributes import (ScalarPix, ScalarLength, QuantityLength,
-                               ScalarSky)
-from ..core.bounding_box import BoundingBox
+from ..core.attributes import (ScalarPixCoord, PositiveScalar,
+                               PositiveScalarAngle, ScalarAngle,
+                               ScalarSkyCoord)
+from ..core.bounding_box import RegionBoundingBox
 from ..core.core import PixelRegion, SkyRegion
 from ..core.mask import RegionMask
 from ..core.metadata import RegionMeta, RegionVisual
@@ -47,7 +48,6 @@ class EllipsePixelRegion(PixelRegion):
 
     Examples
     --------
-
     .. plot::
         :include-source:
 
@@ -56,28 +56,29 @@ class EllipsePixelRegion(PixelRegion):
         from astropy.coordinates import Angle
         from regions import PixCoord, EllipsePixelRegion
         import matplotlib.pyplot as plt
-        x0, y0 = 15, 10
-        a, b = 8, 5
-        theta = Angle(30, 'deg')
-        ell = Ellipse2D(amplitude=100., x_0=x0, y_0=y0, a=a, b=b,
-                        theta=theta.radian)
-        y, x = np.mgrid[0:20, 0:30]
-        fig, ax = plt.subplots(1, 1)
-        ax.imshow(ell(x, y), origin='lower', interpolation='none',
-                  cmap='Greys_r')
 
-        center = PixCoord(x=x0, y=y0)
-        reg = EllipsePixelRegion(center=center, width=2*a, height=2*b,
-                                 angle=theta)
-        patch = reg.as_artist(facecolor='none', edgecolor='red', lw=2)
-        ax.add_patch(patch)
+        fig, ax = plt.subplots(1, 1)
+
+        reg = EllipsePixelRegion(PixCoord(15, 10), width=16, height=10,
+                                 angle=Angle(30, 'deg'))
+        patch = reg.plot(ax=ax, facecolor='none', edgecolor='red', lw=2,
+                         label='Ellipse')
+
+        ax.legend(handles=(patch,), loc='upper center')
+        ax.set_xlim(0, 30)
+        ax.set_ylim(0, 20)
+        ax.set_aspect('equal')
     """
 
     _params = ('center', 'width', 'height', 'angle')
-    center = ScalarPix('center')
-    width = ScalarLength('width')
-    height = ScalarLength('height')
-    angle = QuantityLength('angle')
+    _mpl_artist = 'Patch'
+    center = ScalarPixCoord('The center pixel position as a |PixCoord|.')
+    width = PositiveScalar('The width of the ellipse (before rotation) in '
+                           'pixels as a float.')
+    height = PositiveScalar('The height of the ellipse (before rotation) in '
+                            'pixels as a float.')
+    angle = ScalarAngle('The rotation angle measured anti-clockwise as a '
+                        '|Quantity| angle.')
 
     def __init__(self, center, width, height, angle=0. * u.deg, meta=None,
                  visual=None):
@@ -90,7 +91,6 @@ class EllipsePixelRegion(PixelRegion):
 
     @property
     def area(self):
-        """Region area (float)"""
         return math.pi / 4 * self.width * self.height
 
     def contains(self, pixcoord):
@@ -113,15 +113,19 @@ class EllipsePixelRegion(PixelRegion):
         _, pixscale, north_angle = pixel_scale_angle_at_skycoord(center, wcs)
         height = Angle(self.height * u.pix * pixscale, 'arcsec')
         width = Angle(self.width * u.pix * pixscale, 'arcsec')
-        return EllipseSkyRegion(center, width, height,
-                                angle=self.angle - (north_angle - 90 * u.deg),
-                                meta=self.meta, visual=self.visual)
+        # region sky angles are defined relative to the WCS longitude axis;
+        # photutils aperture sky angles are defined as the PA of the
+        # semimajor axis (i.e., relative to the WCS latitude axis)
+        angle = self.angle - (north_angle - 90 * u.deg)
+        return EllipseSkyRegion(center, width, height, angle=angle,
+                                meta=self.meta.copy(),
+                                visual=self.visual.copy())
 
     @property
     def bounding_box(self):
         """
-        The minimal bounding box (`~regions.BoundingBox`) enclosing the
-        exact elliptical region.
+        The minimal bounding box (`~regions.RegionBoundingBox`)
+        enclosing the exact elliptical region.
         """
         # We use the solution described in
         # https://stackoverflow.com/a/14163413
@@ -139,7 +143,7 @@ class EllipsePixelRegion(PixelRegion):
         ymin = self.center.y - dy
         ymax = self.center.y + dy
 
-        return BoundingBox.from_float(xmin, xmax, ymin, ymax)
+        return RegionBoundingBox.from_float(xmin, xmax, ymin, ymax)
 
     def to_mask(self, mode='center', subpixels=5):
         # NOTE: assumes this class represents a single circle
@@ -202,11 +206,11 @@ class EllipsePixelRegion(PixelRegion):
         # matplotlib expects rotation in degrees (anti-clockwise)
         angle = self.angle.to('deg').value
 
-        mpl_kwargs = self._define_mpl_kwargs(artist='Patch')
+        mpl_kwargs = self.visual.define_mpl_kwargs(self._mpl_artist)
         mpl_kwargs.update(kwargs)
 
         return Ellipse(xy=xy, width=width, height=height, angle=angle,
-                       **kwargs)
+                       **mpl_kwargs)
 
     def _update_from_mpl_selector(self, *args, **kwargs):
         xmin, xmax, ymin, ymax = self._mpl_selector.extents
@@ -240,6 +244,10 @@ class EllipsePixelRegion(PixelRegion):
             region is updated. This only has an effect if ``sync`` is
             `True`. If a callback is set, it is called for the first
             time once the selector has been created.
+        drag_from_anywhere : bool, optional
+            If `True`, the selector can be moved by clicking anywhere within
+            its bounds, else only at the central anchor
+            (only available with matplotlib 3.5 upwards; default: `False`).
         **kwargs : dict
             Additional keyword arguments that are passed to
             `matplotlib.widgets.EllipseSelector`.
@@ -257,14 +265,13 @@ class EllipsePixelRegion(PixelRegion):
         ``selector.set_active(True)`` or ``selector.set_active(False)``.
         """
         from matplotlib.widgets import EllipseSelector
+        from .._utils.optional_deps import MPL_VERSION
 
         if hasattr(self, '_mpl_selector'):
-            raise Exception('Cannot attach more than one selector to a '
-                            'region.')
+            raise AttributeError('Cannot attach more than one selector to a region.')
 
         if self.angle.value != 0:
-            raise NotImplementedError('Cannot create matplotlib selector for '
-                                      'rotated ellipse.')
+            raise NotImplementedError('Cannot create matplotlib selector for rotated ellipse.')
 
         if sync:
             sync_callback = self._update_from_mpl_selector
@@ -272,12 +279,18 @@ class EllipsePixelRegion(PixelRegion):
             def sync_callback(*args, **kwargs):
                 pass
 
-        self._mpl_selector = EllipseSelector(
-            ax, sync_callback, interactive=True,
-            rectprops={'edgecolor': self.visual.get('color', 'black'),
-                       'facecolor': 'none',
-                       'linewidth': self.visual.get('linewidth', 1),
-                       'linestyle': self.visual.get('linestyle', 'solid')})
+        rectprops = {'edgecolor': self.visual.get('color', 'black'),
+                     'facecolor': 'none',
+                     'linewidth': self.visual.get('linewidth', 1),
+                     'linestyle': self.visual.get('linestyle', 'solid')}
+        rectprops.update(kwargs.pop('props', dict()))
+        # `rectprops` renamed `props` in mpl 3.5 and deprecated for 3.7.
+        if MPL_VERSION < 35:
+            kwargs.update({'rectprops': rectprops})
+        else:
+            kwargs.update({'props': rectprops})
+
+        self._mpl_selector = EllipseSelector(ax, sync_callback, interactive=True, **kwargs)
 
         self._mpl_selector.extents = (self.center.x - self.width / 2,
                                       self.center.x + self.width / 2,
@@ -295,7 +308,7 @@ class EllipsePixelRegion(PixelRegion):
         """
         Rotate the region.
 
-        Postive ``angle`` corresponds to counter-clockwise rotation.
+        Positive ``angle`` corresponds to counter-clockwise rotation.
 
         Parameters
         ----------
@@ -338,10 +351,13 @@ class EllipseSkyRegion(SkyRegion):
     """
 
     _params = ('center', 'width', 'height', 'angle')
-    center = ScalarSky('center')
-    width = QuantityLength('width')
-    height = QuantityLength('height')
-    angle = QuantityLength('angle')
+    center = ScalarSkyCoord('The center position as a |SkyCoord|.')
+    width = PositiveScalarAngle('The width of the ellipse (before rotation) '
+                                'as a |Quantity| angle.')
+    height = PositiveScalarAngle('The height of the ellipse (before rotation) '
+                                 'as a |Quantity| angle.')
+    angle = ScalarAngle('The rotation angle measured anti-clockwise as a '
+                        '|Quantity| angle.')
 
     def __init__(self, center, width, height, angle=0. * u.deg, meta=None,
                  visual=None):
@@ -357,6 +373,10 @@ class EllipseSkyRegion(SkyRegion):
             self.center, wcs)
         height = (self.height / pixscale).to(u.pixel).value
         width = (self.width / pixscale).to(u.pixel).value
+        # region sky angles are defined relative to the WCS longitude axis;
+        # photutils aperture sky angles are defined as the PA of the
+        # semimajor axis (i.e., relative to the WCS latitude axis)
         angle = self.angle + (north_angle - 90 * u.deg)
         return EllipsePixelRegion(center, width, height, angle=angle,
-                                  meta=self.meta, visual=self.visual)
+                                  meta=self.meta.copy(),
+                                  visual=self.visual.copy())
