@@ -8,9 +8,10 @@ import astropy.units as u
 from astropy.wcs.utils import pixel_to_skycoord
 import numpy as np
 
-from ..core.attributes import (ScalarPix, ScalarLength, QuantityLength,
-                               ScalarSky)
-from ..core.bounding_box import BoundingBox
+from ..core.attributes import (ScalarPixCoord, PositiveScalar,
+                               PositiveScalarAngle, ScalarAngle,
+                               ScalarSkyCoord)
+from ..core.bounding_box import RegionBoundingBox
 from ..core.core import PixelRegion, SkyRegion
 from ..core.mask import RegionMask
 from ..core.metadata import RegionMeta, RegionVisual
@@ -54,28 +55,28 @@ class RectanglePixelRegion(PixelRegion):
         from regions import PixCoord, RectanglePixelRegion
         import matplotlib.pyplot as plt
 
-        x, y = 15, 10
-        width, height = 8, 5
-        angle = Angle(30, 'deg')
-
         fig, ax = plt.subplots(1, 1)
 
-        center = PixCoord(x=x, y=y)
-        reg = RectanglePixelRegion(center=center, width=width,
-                                   height=height, angle=angle)
-        patch = reg.as_artist(facecolor='none', edgecolor='red', lw=2)
-        ax.add_patch(patch)
+        reg = RectanglePixelRegion(PixCoord(x=15, y=10), width=8,
+                                   height=5, angle=Angle(30, 'deg'))
+        patch = reg.plot(ax=ax, facecolor='none', edgecolor='red', lw=2,
+                         label='Rectangle')
 
-        plt.xlim(0, 30)
-        plt.ylim(0, 20)
+        ax.legend(handles=(patch,), loc='upper center')
+        ax.set_xlim(0, 30)
+        ax.set_ylim(0, 20)
         ax.set_aspect('equal')
     """
 
     _params = ('center', 'width', 'height', 'angle')
-    center = ScalarPix('center')
-    width = ScalarLength('width')
-    height = ScalarLength('height')
-    angle = QuantityLength('angle')
+    _mpl_artist = 'Patch'
+    center = ScalarPixCoord('The center pixel position as a |PixCoord|.')
+    width = PositiveScalar('The width of the rectangle (before rotation) in '
+                           'pixels as a float.')
+    height = PositiveScalar('The height of the rectangle (before rotation) '
+                            'in pixels as a float.')
+    angle = ScalarAngle('The rotation angle measured anti-clockwise as a '
+                        '|Quantity| angle.')
 
     def __init__(self, center, width, height, angle=0 * u.deg, meta=None,
                  visual=None):
@@ -109,9 +110,13 @@ class RectanglePixelRegion(PixelRegion):
         _, pixscale, north_angle = pixel_scale_angle_at_skycoord(center, wcs)
         width = Angle(self.width * u.pix * pixscale, 'arcsec')
         height = Angle(self.height * u.pix * pixscale, 'arcsec')
+        # region sky angles are defined relative to the WCS longitude axis;
+        # photutils aperture sky angles are defined as the PA of the
+        # semimajor axis (i.e., relative to the WCS latitude axis)
         angle = self.angle - (north_angle - 90 * u.deg)
         return RectangleSkyRegion(center, width, height, angle=angle,
-                                  meta=self.meta, visual=self.visual)
+                                  meta=self.meta.copy(),
+                                  visual=self.visual.copy())
 
     @property
     def bounding_box(self):
@@ -131,7 +136,7 @@ class RectanglePixelRegion(PixelRegion):
         ymin = self.center.y - dy
         ymax = self.center.y + dy
 
-        return BoundingBox.from_float(xmin, xmax, ymin, ymax)
+        return RegionBoundingBox.from_float(xmin, xmax, ymin, ymax)
 
     def to_mask(self, mode='center', subpixels=5):
         # NOTE: assumes this class represents a single rectangle
@@ -195,7 +200,7 @@ class RectanglePixelRegion(PixelRegion):
         # matplotlib expects rotation in degrees (anti-clockwise)
         angle = self.angle.to('deg').value
 
-        mpl_kwargs = self._define_mpl_kwargs(artist='Patch')
+        mpl_kwargs = self.visual.define_mpl_kwargs(self._mpl_artist)
         mpl_kwargs.update(kwargs)
 
         return Rectangle(xy=xy, width=width, height=height,
@@ -233,6 +238,10 @@ class RectanglePixelRegion(PixelRegion):
             region is updated. This only has an effect if ``sync`` is
             `True`. If a callback is set, it is called for the first
             time once the selector has been created.
+        drag_from_anywhere : bool, optional
+            If `True`, the selector can be moved by clicking anywhere within
+            its bounds, else only at the central anchor
+            (only available with matplotlib 3.5 upwards; default: `False`).
         **kwargs : dict
             Additional keyword arguments are passed to
             `matplotlib.widgets.RectangleSelector`.
@@ -250,10 +259,10 @@ class RectanglePixelRegion(PixelRegion):
         ``selector.set_active(True)`` or ``selector.set_active(False)``.
         """
         from matplotlib.widgets import RectangleSelector
+        from .._utils.optional_deps import MPL_VERSION
 
         if hasattr(self, '_mpl_selector'):
-            raise Exception('Cannot attach more than one selector to a '
-                            'region.')
+            raise AttributeError('Cannot attach more than one selector to a region.')
 
         if self.angle.value != 0:
             raise NotImplementedError('Cannot create matplotlib selector for '
@@ -265,12 +274,18 @@ class RectanglePixelRegion(PixelRegion):
             def sync_callback(*args, **kwargs):
                 pass
 
-        self._mpl_selector = RectangleSelector(
-            ax, sync_callback, interactive=True,
-            rectprops={'edgecolor': self.visual.get('color', 'black'),
-                       'facecolor': 'none',
-                       'linewidth': self.visual.get('linewidth', 1),
-                       'linestyle': self.visual.get('linestyle', 'solid')})
+        rectprops = {'edgecolor': self.visual.get('color', 'black'),
+                     'facecolor': 'none',
+                     'linewidth': self.visual.get('linewidth', 1),
+                     'linestyle': self.visual.get('linestyle', 'solid')}
+        rectprops.update(kwargs.pop('props', dict()))
+        # `rectprops` renamed `props` in mpl 3.5 and deprecated for 3.7.
+        if MPL_VERSION < 35:
+            kwargs.update({'rectprops': rectprops})
+        else:
+            kwargs.update({'props': rectprops})
+
+        self._mpl_selector = RectangleSelector(ax, sync_callback, interactive=True, **kwargs)
 
         self._mpl_selector.extents = (self.center.x - self.width / 2,
                                       self.center.x + self.width / 2,
@@ -306,8 +321,8 @@ class RectanglePixelRegion(PixelRegion):
         """
         x, y = self.corners.T
         vertices = PixCoord(x=x, y=y)
-        return PolygonPixelRegion(vertices=vertices, meta=self.meta,
-                                  visual=self.visual)
+        return PolygonPixelRegion(vertices=vertices, meta=self.meta.copy(),
+                                  visual=self.visual.copy())
 
     def _lower_left_xy(self):
         """
@@ -333,7 +348,7 @@ class RectanglePixelRegion(PixelRegion):
         """
         Rotate the region.
 
-        Postive ``angle`` corresponds to counter-clockwise rotation.
+        Positive ``angle`` corresponds to counter-clockwise rotation.
 
         Parameters
         ----------
@@ -376,10 +391,13 @@ class RectangleSkyRegion(SkyRegion):
     """
 
     _params = ('center', 'width', 'height', 'angle')
-    center = ScalarSky('center')
-    width = QuantityLength('width')
-    height = QuantityLength('height')
-    angle = QuantityLength('angle')
+    center = ScalarSkyCoord('The center position as a |SkyCoord|.')
+    width = PositiveScalarAngle('The width of the rectangle (before rotation) '
+                                'as a |Quantity| angle.')
+    height = PositiveScalarAngle('The height of the rectangle (before '
+                                 'rotation) as a |Quantity| angle.')
+    angle = ScalarAngle('The rotation angle measured anti-clockwise as a '
+                        '|Quantity| angle.')
 
     def __init__(self, center, width, height, angle=0 * u.deg, meta=None,
                  visual=None):
@@ -395,6 +413,10 @@ class RectangleSkyRegion(SkyRegion):
             self.center, wcs)
         width = (self.width / pixscale).to(u.pix).value
         height = (self.height / pixscale).to(u.pix).value
+        # region sky angles are defined relative to the WCS longitude axis;
+        # photutils aperture sky angles are defined as the PA of the
+        # semimajor axis (i.e., relative to the WCS latitude axis)
         angle = self.angle + (north_angle - 90 * u.deg)
         return RectanglePixelRegion(center, width, height, angle=angle,
-                                    meta=self.meta, visual=self.visual)
+                                    meta=self.meta.copy(),
+                                    visual=self.visual.copy())
