@@ -11,11 +11,11 @@ from astropy.tests.helper import assert_quantity_allclose
 from astropy.utils.data import get_pkg_data_filename
 from astropy.wcs import WCS
 
-from ...core import PixCoord
+from ...core import PixCoord, RegionMeta, RegionVisual
 from ...tests.helpers import make_simple_wcs
+from ..._utils.optional_deps import HAS_MATPLOTLIB, MPL_VERSION  # noqa
 from ..rectangle import RectanglePixelRegion, RectangleSkyRegion
 from .test_common import BaseTestPixelRegion, BaseTestSkyRegion
-from .utils import HAS_MATPLOTLIB  # noqa
 
 
 @pytest.fixture(scope='session', name='wcs')
@@ -55,9 +55,10 @@ def test_corners():
 
 
 class TestRectanglePixelRegion(BaseTestPixelRegion):
-
+    meta = RegionMeta({'text': 'test'})
+    visual = RegionVisual({'color': 'blue'})
     reg = RectanglePixelRegion(center=PixCoord(3, 4), width=4, height=3,
-                               angle=5 * u.deg)
+                               angle=5 * u.deg, meta=meta, visual=visual)
     sample_box = [-2, 8, -1, 9]
     inside = [(4.5, 4)]
     outside = [(5, 2.5)]
@@ -73,8 +74,8 @@ class TestRectanglePixelRegion(BaseTestPixelRegion):
         assert reg.width == 4
         assert reg.height == 3
         assert_allclose(reg.angle.to_value("deg"), 5)
-        assert reg.visual == {}
-        assert reg.meta == {}
+        assert reg.meta == self.meta
+        assert reg.visual == self.visual
 
     def test_pix_sky_roundtrip(self):
         wcs = make_simple_wcs(SkyCoord(2 * u.deg, 3 * u.deg), 0.1 * u.deg, 20)
@@ -84,6 +85,14 @@ class TestRectanglePixelRegion(BaseTestPixelRegion):
         assert_allclose(reg_new.width, self.reg.width)
         assert_allclose(reg_new.height, self.reg.height)
         assert_quantity_allclose(reg_new.angle, self.reg.angle)
+        assert reg_new.meta == self.reg.meta
+        assert reg_new.visual == self.reg.visual
+
+        # test that converted region meta and visual are copies and not views
+        reg_new.meta['text'] = 'new'
+        reg_new.visual['color'] = 'green'
+        assert reg_new.meta['text'] != self.reg.meta['text']
+        assert reg_new.visual['color'] != self.reg.visual['color']
 
     @pytest.mark.skipif('not HAS_MATPLOTLIB')
     def test_as_artist(self):
@@ -100,9 +109,17 @@ class TestRectanglePixelRegion(BaseTestPixelRegion):
         assert_allclose(reg.center.xy, (1, 4))
         assert_allclose(reg.angle.to_value('deg'), 95)
 
+    def test_eq(self):
+        reg = self.reg.copy()
+        assert reg == self.reg
+        reg.angle = 35 * u.deg
+        assert reg != self.reg
+
+    @pytest.mark.skipif(MPL_VERSION < 33, reason='requires `do_event`')
     @pytest.mark.parametrize('sync', (False, True))
     def test_as_mpl_selector(self, sync):
         plt = pytest.importorskip('matplotlib.pyplot')
+        from matplotlib.testing.widgets import do_event
 
         data = np.random.random((16, 16))
         mask = np.zeros_like(data)
@@ -111,39 +128,23 @@ class TestRectanglePixelRegion(BaseTestPixelRegion):
         ax.imshow(data)
 
         def update_mask(reg):
-            mask[:] = reg.to_mask(
-                mode='subpixels', subpixels=10).to_image(data.shape)
+            mask[:] = reg.to_mask(mode='subpixels', subpixels=10).to_image(data.shape)
 
         # For now this will only work with unrotated rectangles. Once
         # this works with rotated rectangles, the following exception
         # check can be removed as well as the ``angle=0 * u.deg`` in the
         # call to copy() below.
         with pytest.raises(NotImplementedError,
-                           match=('Cannot create matplotlib selector for '
-                                  'rotated rectangle.')):
+                           match=('Cannot create matplotlib selector for rotated rectangle.')):
             self.reg.as_mpl_selector(ax)
 
         region = self.reg.copy(angle=0 * u.deg)
 
         selector = region.as_mpl_selector(ax, callback=update_mask, sync=sync)  # noqa
 
-        from matplotlib.backend_bases import MouseEvent, MouseButton
-
-        x, y = ax.transData.transform([[7.3, 4.4]])[0]
-        ax.figure.canvas.callbacks.process('button_press_event',
-                                           MouseEvent('button_press_event',
-                                                      ax.figure.canvas, x, y,
-                                                      button=MouseButton.LEFT))
-        x, y = ax.transData.transform([[9.3, 5.4]])[0]
-        ax.figure.canvas.callbacks.process('motion_notify_event',
-                                           MouseEvent('button_press_event',
-                                                      ax.figure.canvas, x, y,
-                                                      button=MouseButton.LEFT))
-        x, y = ax.transData.transform([[9.3, 5.4]])[0]
-        ax.figure.canvas.callbacks.process('button_release_event',
-                                           MouseEvent('button_press_event',
-                                                      ax.figure.canvas, x, y,
-                                                      button=MouseButton.LEFT))
+        do_event(selector, 'press', xdata=7.3, ydata=4.4, button=1)
+        do_event(selector, 'onmove', xdata=9.3, ydata=5.4, button=1)
+        do_event(selector, 'release', xdata=9.3, ydata=5.4, button=1)
 
         ax.figure.canvas.draw()
 
@@ -154,8 +155,7 @@ class TestRectanglePixelRegion(BaseTestPixelRegion):
             assert_allclose(region.height, 1)
             assert_quantity_allclose(region.angle, 0 * u.deg)
 
-            assert_equal(mask, region.to_mask(
-                mode='subpixels', subpixels=10).to_image(data.shape))
+            assert_equal(mask, region.to_mask(mode='subpixels', subpixels=10).to_image(data.shape))
 
         else:
             assert_allclose(region.center.x, 3)
@@ -166,10 +166,109 @@ class TestRectanglePixelRegion(BaseTestPixelRegion):
 
             assert_equal(mask, 0)
 
-        with pytest.raises(Exception, match=('Cannot attach more than one '
-                                             'selector to a region.')):
+        with pytest.raises(AttributeError, match=('Cannot attach more than one selector to a reg')):
             region.as_mpl_selector(ax)
 
+    @pytest.mark.skipif(MPL_VERSION < 33, reason='requires `do_event`')
+    @pytest.mark.parametrize('anywhere', (False, True))
+    def test_mpl_selector_drag(self, anywhere):
+        """Test dragging of entire region from central handle and anywhere."""
+
+        plt = pytest.importorskip('matplotlib.pyplot')
+        from matplotlib.testing.widgets import do_event  # click_and_drag  # MPL_VERSION >= 36
+
+        data = np.random.random((16, 16))
+        mask = np.zeros_like(data)
+
+        ax = plt.subplot(1, 1, 1)
+        ax.imshow(data)
+
+        def update_mask(reg):
+            mask[:] = reg.to_mask(mode='subpixels', subpixels=10).to_image(data.shape)
+
+        region = self.reg.copy(angle=0 * u.deg)
+
+        if anywhere and MPL_VERSION < 35:
+            pytest.skip('Requires `drag_from_anywhere` kwarg')
+        elif MPL_VERSION < 35:
+            selector = region.as_mpl_selector(ax, callback=update_mask)
+        else:
+            selector = region.as_mpl_selector(ax, callback=update_mask, drag_from_anywhere=anywhere)
+            assert selector.drag_from_anywhere is anywhere
+            assert region._mpl_selector.drag_from_anywhere is anywhere
+
+        # click_and_drag(selector, start=(3, 4), end=(3.5, 4.5))
+        do_event(selector, 'press', xdata=3, ydata=4, button=1)
+        do_event(selector, 'onmove', xdata=3.5, ydata=4.5, button=1)
+        do_event(selector, 'release', xdata=3.5, ydata=4.5, button=1)
+
+        ax.figure.canvas.draw()
+
+        assert_allclose(region.center.x, 3.5)
+        assert_allclose(region.center.y, 4.5)
+        assert_allclose(region.width, 4)
+        assert_allclose(region.height, 3)
+
+        do_event(selector, 'press', xdata=3.25, ydata=4.25, button=1)
+        do_event(selector, 'onmove', xdata=4.25, ydata=5.25, button=1)
+        do_event(selector, 'release', xdata=4.25, ydata=5.25, button=1)
+
+        ax.figure.canvas.draw()
+
+        # For drag_from_anywhere=False this will have created a new 1x1 rectangle.
+        if anywhere:
+            assert_allclose(region.center.x, 4.5)
+            assert_allclose(region.center.y, 5.5)
+            assert_allclose(region.width, 4)
+            assert_allclose(region.height, 3)
+        else:
+            assert_allclose(region.center.x, 4.5)
+            assert_allclose(region.center.y, 5.5)
+
+        assert_equal(mask, region.to_mask(mode='subpixels', subpixels=10).to_image(data.shape))
+
+    @pytest.mark.parametrize('userargs',
+                             ({'useblit': True},
+                              {'grab_range': 20, 'minspanx': 5,  'minspany': 4},
+                              {'props': {'facecolor': 'blue', 'linewidth': 2}},
+                              {'twit': 'gumby'}))
+    def test_mpl_selector_kwargs(self, userargs):
+        """Test that additional kwargs are passed to selector."""
+
+        plt = pytest.importorskip('matplotlib.pyplot')
+
+        data = np.random.random((16, 16))
+        mask = np.zeros_like(data)
+
+        ax = plt.subplot(1, 1, 1)
+        ax.imshow(data)
+
+        def update_mask(reg):
+            mask[:] = reg.to_mask(mode='subpixels', subpixels=10).to_image(data.shape)
+
+        region = self.reg.copy(angle=0 * u.deg)
+
+        if MPL_VERSION < 35 and 'grab_range' in userargs:
+            userargs['maxdist'] = userargs.pop('grab_range')
+
+        if 'twit' in userargs:
+            with pytest.raises(TypeError, match=(r'__init__.. got an unexpected keyword argument')):
+                selector = region.as_mpl_selector(ax, callback=update_mask, **userargs)
+        else:
+            selector = region.as_mpl_selector(ax, callback=update_mask, **userargs)
+            assert region._mpl_selector.artists[0].get_edgecolor() == (0, 0, 1, 1)
+
+            if 'props' in userargs:
+                assert region._mpl_selector.artists[0].get_facecolor() == (0, 0, 1, 1)
+                assert region._mpl_selector.artists[0].get_linewidth() == 2
+            else:
+                assert region._mpl_selector.artists[0].get_facecolor() == (0, 0, 0, 0)
+                assert region._mpl_selector.artists[0].get_linewidth() == 1
+
+                for key, val in userargs.items():
+                    assert getattr(region._mpl_selector, key) == val
+                    assert getattr(selector, key) == val
+
 
 def test_rectangular_pixel_region_bbox():
     # odd sizes
@@ -204,10 +303,11 @@ def test_rectangular_pixel_region_bbox():
 
 
 class TestRectangleSkyRegion(BaseTestSkyRegion):
-
+    meta = RegionMeta({'text': 'test'})
+    visual = RegionVisual({'color': 'blue'})
     reg = RectangleSkyRegion(center=SkyCoord(3, 4, unit='deg'),
                              width=4 * u.deg, height=3 * u.deg,
-                             angle=5 * u.deg)
+                             angle=5 * u.deg, meta=meta, visual=visual)
 
     expected_repr = ('<RectangleSkyRegion(center=<SkyCoord (ICRS): (ra, dec) '
                      'in deg\n    (3., 4.)>, width=4.0 deg, height=3.0 deg, '
@@ -222,11 +322,17 @@ class TestRectangleSkyRegion(BaseTestSkyRegion):
         assert_allclose(reg.width.to_value('deg'), 4)
         assert_allclose(reg.height.to_value('deg'), 3)
         assert_allclose(reg.angle.to_value('deg'), 5)
-        assert reg.visual == {}
-        assert reg.meta == {}
+        assert reg.meta == self.meta
+        assert reg.visual == self.visual
 
     def test_contains(self, wcs):
         position = SkyCoord([1, 3] * u.deg, [2, 4] * u.deg)
         # 1,2 is outside, 3,4 is the center and is inside
         assert all(self.reg.contains(position, wcs)
                    == np.array([False, True], dtype='bool'))
+
+    def test_eq(self):
+        reg = self.reg.copy()
+        assert reg == self.reg
+        reg.angle = 10 * u.deg
+        assert reg != self.reg
